@@ -43,19 +43,24 @@ def match_vlan_interface(device, job_def):
         return False
     interfaces = []
     logger = logging.getLogger('dispatcher-master')
+    device_dict = DeviceDictionary.get(device.hostname).to_dict()
+    if 'tags' not in device_dict['parameters']:
+        logger.error("%s has no tags in the device dictionary parameters", device.hostname)
+        return False
     for vlan_name in job_def['protocols']['lava-vland']:
         tag_list = job_def['protocols']['lava-vland'][vlan_name]['tags']
-        device_dict = DeviceDictionary.get(device.hostname).to_dict()
-        if 'tags' not in device_dict['parameters']:
-            return False
         for interface, tags in device_dict['parameters']['tags'].iteritems():
+            logger.info(
+                "Job requests %s for %s, device %s provides %s for %s",
+                tag_list, vlan_name, device.hostname, tags, interface)
             # tags & job tags must equal job tags
             # device therefore must support all job tags, not all job tags available on the device need to be specified
             if set(tags) & set(tag_list) == set(tag_list) and interface not in interfaces:
-                logger.debug("Matched vlan %s to interface %s on %s", vlan_name, interface, device)
+                logger.info("Matched vlan %s to interface %s on %s", vlan_name, interface, device)
                 interfaces.append(interface)
                 # matched, do not check any further interfaces of this device for this vlan
                 break
+    logger.info("Matched: %s" % (len(interfaces) == len(job_def['protocols']['lava-vland'].keys())))
     return len(interfaces) == len(job_def['protocols']['lava-vland'].keys())
 
 
@@ -520,8 +525,9 @@ def assign_jobs():
             if job.is_pipeline:
                 job_dict = yaml.load(job.definition)
                 if 'protocols' in job_dict and 'lava-vland' in job_dict['protocols']:
+                    logger.info("[%d] checking vlan interface support", job.id)
                     if not match_vlan_interface(device, job_dict):
-                        logger.debug("%s does not match vland tags", str(device.hostname))
+                        logger.info("%s does not match vland tags", str(device.hostname))
                         if device in devices:
                             devices.remove(device)
                         continue
@@ -787,54 +793,46 @@ def select_device(job, dispatchers):  # pylint: disable=too-many-return-statemen
             device_object.target = device.hostname
         device_object['hostname'] = device.hostname
 
-    validate_list = job.sub_jobs_list if job.is_multinode else [job]
-    for check_job in validate_list:
-        # dynamic connections still get the device config of the host
+    # write the pipeline description to the job output directory.
+    # Can only validate this job for this device, even with multinode
+    # dynamic connections still get the device config of the host
+    try:
+        logger.info("[%d] Parsing definition", job.id)
+        # pass (unused) output_dir just for validation as there is no zmq socket either.
+        pipeline_job = parser.parse(
+            job.definition, device_object,
+            job.id, None, None, None, output_dir=job.output_dir)
+    except (
+            AttributeError, JobError, NotImplementedError,
+            KeyError, TypeError, RuntimeError) as exc:
+        exc = format_exc(exc)
+        logger.error('[%d] parser error: %s', job.id, exc)
+        fail_job(job, fail_msg=exc)
+        return None
+
+    pipeline = pipeline_job.describe()
+    if not os.path.exists(job.output_dir):
+        os.makedirs(job.output_dir)
+    pipeline_dump = yaml.dump(pipeline)
+    with open(os.path.join(job.output_dir, 'description.yaml'), 'w') as describe_yaml:
+        describe_yaml.write(pipeline_dump)
+    if not map_metadata(pipeline_dump, job):
+        logger.warning("[%d] unable to map metadata", job.id)
+
+    # add the compatibility result from the master to the definition for comparison on the slave.
+    if 'compatibility' in pipeline:
         try:
-            logger.info("[%d] Parsing definition", check_job.id)
-            # pass (unused) output_dir just for validation as there is no zmq socket either.
-            pipeline_job = parser.parse(
-                check_job.definition, device_object,
-                check_job.id, None, None, None, output_dir=check_job.output_dir)
-        except (
-                AttributeError, JobError, NotImplementedError,
-                KeyError, TypeError, RuntimeError) as exc:
-            exc = format_exc(exc)
-            logger.error('[%d] parser error: %s', check_job.id, exc)
-            fail_job(check_job, fail_msg=exc)
-            return None
-        try:
-            logger.info("[%d] Validating actions", check_job.id)
-            pipeline_job.pipeline.validate_actions()
-        except (AttributeError, JobError, KeyError, TypeError, RuntimeError) as exc:
-            exc = format_exc(exc)
-            logger.error({device: exc})
-            fail_job(check_job, fail_msg=exc)
-            return None
-        if pipeline_job:
-            pipeline = pipeline_job.describe()
-            # write the pipeline description to the job output directory.
-            if not os.path.exists(check_job.output_dir):
-                os.makedirs(check_job.output_dir)
-            pipeline_dump = yaml.dump(pipeline)
-            with open(os.path.join(check_job.output_dir, 'description.yaml'), 'w') as describe_yaml:
-                describe_yaml.write(pipeline_dump)
-            if not map_metadata(pipeline_dump, check_job):
-                logger.warning("[%d] unable to map metadata", check_job.id)
-            # add the compatibility result from the master to the definition for comparison on the slave.
-            if 'compatibility' in pipeline:
-                try:
-                    compat = int(pipeline['compatibility'])
-                except ValueError:
-                    logger.error("[%d] Unable to parse job compatibility: %s",
-                                 check_job.id, pipeline['compatibility'])
-                    compat = 0
-                check_job.pipeline_compatibility = compat
-                check_job.save(update_fields=['pipeline_compatibility'])
-            else:
-                logger.error("[%d] Unable to identify job compatibility.", check_job.id)
-                fail_job(check_job, fail_msg='Unknown compatibility')
-                return None
+            compat = int(pipeline['compatibility'])
+        except ValueError:
+            logger.error("[%d] Unable to parse job compatibility: %s",
+                         job.id, pipeline['compatibility'])
+            compat = 0
+        job.pipeline_compatibility = compat
+        job.save(update_fields=['pipeline_compatibility'])
+    else:
+        logger.error("[%d] Unable to identify job compatibility.", job.id)
+        fail_job(job, fail_msg='Unknown compatibility')
+        return None
 
     return device
 
